using System;
using System.Drawing;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using AddinExpress.OL;
using Outlook = Microsoft.Office.Interop.Outlook;

namespace adx_ol_cs_OutlookRecipients
{
    public partial class frmRecipients : AddinExpress.OL.ADXOlForm
    {
        public frmRecipients()
        {
            InitializeComponent();
        }

        private void frmRecipients_ADXNewInspector(object sender, ADXNewInspectorEventArgs args)
        {
            Outlook._Inspector currInsp = null;
            Outlook._MailItem mail = null;
            object item = null;

            try
            {
                currInsp = this.InspectorObj as Outlook._Inspector;
                item = currInsp.CurrentItem;
                if (item is Outlook._MailItem)
                {
                    mail = item as Outlook._MailItem;
                    if (mail.Sent)
                    {
                        txtFrom.Text = mail.SenderName;
                        GetRecipients(mail, Outlook.OlMailRecipientType.olTo, lbTo.Items);
                        GetRecipients(mail, Outlook.OlMailRecipientType.olCC, lbCC.Items);
                    }
                    else
                    {
                        lblFrom.Visible = false;
                        txtFrom.Visible = false;
                    }
                }
            }
            finally
            {
                if (item != null) Marshal.ReleaseComObject(item);
                if (currInsp != null) Marshal.ReleaseComObject(currInsp);
            }
        }

        private void lblAddTo_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            AddRecipient(Outlook.OlMailRecipientType.olTo);
        }

        private void lblAddCC_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            AddRecipient(Outlook.OlMailRecipientType.olCC);
        }

        private void lblAddBCC_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            AddRecipient(Outlook.OlMailRecipientType.olBCC);
        }

        private void AddRecipient(Outlook.OlMailRecipientType recipientType)
        {
            frmAddRecipient frmRecipient = new frmAddRecipient();
            if (frmRecipient != null)
                try
                {
                    Outlook._MailItem mailItem = (InspectorObj as Outlook._Inspector).CurrentItem as Outlook._MailItem;
                    if (mailItem != null)
                        try
                        {
                            if (frmRecipient.ShowDialog() == DialogResult.OK)
                            {
                                string fullDisplayName;
                                if (frmRecipient.txtEmail.Text != String.Empty)
                                {
                                    fullDisplayName = frmRecipient.txtName.Text + " (" + frmRecipient.txtEmail.Text + ")";
                                }
                                else
                                {
                                    fullDisplayName = frmRecipient.txtName.Text;
                                }
                                Outlook.Recipient recipient = AddRecipientToMailItem(mailItem, fullDisplayName, recipientType);
                                if (recipient != null)
                                    try
                                    {
                                        if (recipientType == Outlook.OlMailRecipientType.olTo)
                                            AddRecipientToListBox(recipient, lbTo.Items);
                                        else if (recipientType == Outlook.OlMailRecipientType.olCC)
                                            AddRecipientToListBox(recipient, lbCC.Items);
                                        else if (recipientType == Outlook.OlMailRecipientType.olBCC)
                                            AddRecipientToListBox(recipient, lbBCC.Items);
                                    }
                                    finally
                                    {
                                        Marshal.ReleaseComObject(recipient);
                                    }
                                else
                                    System.Windows.Forms.MessageBox.Show(
                                        "The resolve method returned false!" + Environment.NewLine +
                                        "Please check the e-mail address and name you entered.");
                            }
                        }
                        finally
                        {
                            Marshal.ReleaseComObject(mailItem);
                        }
                }
                finally
                {
                    frmRecipient.Dispose();
                }
        }

        private void AddRecipientToListBox(Outlook.Recipient recipient, ListBox.ObjectCollection listBoxItems)
        {
            Outlook.AddressEntry entry = recipient.AddressEntry;
            if (entry != null)
                try
                {
                    if (entry.Name.IndexOf("@") < 0)
                        listBoxItems.Add(entry.Name + " (" + entry.Address + ")");
                    else
                        listBoxItems.Add(entry.Name);
                }
                finally
                {
                    Marshal.ReleaseComObject(entry);
                }
        }

        private Outlook.Recipient AddRecipientToMailItem(Outlook._MailItem mail, string fullName, Outlook.OlMailRecipientType recipientType)
        {
            Outlook.Recipients recipients = mail.Recipients as Outlook.Recipients;
            if (recipients != null)
                try
                {
                    Outlook.Recipient recipient = recipients.Add(fullName) as Outlook.Recipient;
                    if (recipient != null)
                    {
                        recipient.Resolve();
                        if (recipient.Resolved)
                        {
                            recipient.Type = (int)recipientType;
                            recipients.ResolveAll();
                            return recipient;
                        }
                        else
                        {
                            recipient.Delete();
                            return null;
                        }
                    }
                    return null;
                }
                finally
                {
                    Marshal.ReleaseComObject(recipients);
                }
            else
                return null;
        }

        private void GetRecipients(Outlook._MailItem mail, Outlook.OlMailRecipientType recipientType, ListBox.ObjectCollection listBoxItems)
        {
            listBoxItems.Clear();
            Outlook.Recipients recipients = mail.Recipients;
            if (recipients != null)
                try
                {
                    for (int i = 1; i <= recipients.Count; i++)
                    {
                        Outlook.Recipient recipient = recipients[i];
                        if (recipient != null)
                            try
                            {
                                if (recipient.Type == (int)recipientType)
                                {
                                    Outlook.AddressEntry entry = recipient.AddressEntry;
                                    if (entry != null)
                                        try
                                        {
                                            if (entry.Name.IndexOf("@") < 0)
                                                listBoxItems.Add(entry.Name + " (" + entry.Address + ")");
                                            else
                                                listBoxItems.Add(entry.Name);
                                        }
                                        finally
                                        {
                                            Marshal.ReleaseComObject(entry);
                                        }
                                }
                            }
                            finally
                            {
                                Marshal.ReleaseComObject(recipient);
                            }
                    }
                }
                finally
                {
                    Marshal.ReleaseComObject(recipients);
                }
        }

        private void DeleteRecipient(Outlook._MailItem mailItem, Outlook.OlMailRecipientType recipientType, string recipientFullName)
        {
            Outlook.Recipients recipients = mailItem.Recipients as Outlook.Recipients;
            if (recipients != null)
                try
                {
                    string fullName;
                    for (int i = recipients.Count; i >= 1; i--)
                    {
                        Outlook.Recipient recipient = recipients[i];
                        if (recipient != null)
                            try
                            {
                                if (recipient.Type == (int)recipientType)
                                {
                                    Outlook.AddressEntry entry = recipient.AddressEntry;
                                    if (entry != null)
                                        try
                                        {
                                            if (entry.Name.IndexOf("@") < 0)
                                                fullName = entry.Name + " (" + entry.Address + ")";
                                            else
                                                fullName = entry.Name;
                                            if (fullName.Equals(recipientFullName))
                                            {
                                                recipients.Remove(i);
                                                break;
                                            }
                                        }
                                        finally
                                        {
                                            Marshal.ReleaseComObject(entry);
                                        }
                                }
                            }
                            finally
                            {
                                Marshal.ReleaseComObject(recipient);
                            }
                    }
                }
                finally
                {
                    Marshal.ReleaseComObject(recipients);
                }
        }

        private void RefreshListBox(Outlook._MailItem mail)
        {
            GetRecipients(mail, Outlook.OlMailRecipientType.olTo, lbTo.Items);
            GetRecipients(mail, Outlook.OlMailRecipientType.olCC, lbCC.Items);
            GetRecipients(mail, Outlook.OlMailRecipientType.olBCC, lbBCC.Items);
        }

        private void lblDeleteTo_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            Outlook._Inspector currInsp = null;
            Outlook._MailItem mail = null;
            object item = null;

            try
            {
                currInsp = this.InspectorObj as Outlook.Inspector;
                item = currInsp.CurrentItem;
                if (item is Outlook._MailItem)
                {
                    mail = item as Outlook._MailItem;
                    DeleteRecipient(mail, Outlook.OlMailRecipientType.olTo, lbTo.Text);
                    RefreshListBox(mail);
                }

            }
            finally
            {
                if (item != null) Marshal.ReleaseComObject(item);
            }
        }

        private void lblDeleteCC_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            Outlook._Inspector currInsp = null;
            Outlook._MailItem mail = null;
            object item = null;

            try
            {
                currInsp = this.InspectorObj as Outlook.Inspector;
                item = currInsp.CurrentItem;
                if (item is Outlook._MailItem)
                {
                    mail = item as Outlook._MailItem;
                    DeleteRecipient(mail, Outlook.OlMailRecipientType.olCC, lbTo.Text);
                    RefreshListBox(mail);
                }

            }
            finally
            {
                if (item != null) Marshal.ReleaseComObject(item);
            }
        }

        private void lblDeleteBCC_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            Outlook._Inspector currInsp = null;
            Outlook._MailItem mail = null;
            object item = null;

            try
            {
                currInsp = this.InspectorObj as Outlook.Inspector;
                item = currInsp.CurrentItem;
                if (item is Outlook._MailItem)
                {
                    mail = item as Outlook._MailItem;
                    DeleteRecipient(mail, Outlook.OlMailRecipientType.olBCC, lbTo.Text);
                    RefreshListBox(mail);
                }

            }
            finally
            {
                if (item != null) Marshal.ReleaseComObject(item);
            }
        }

        private void btnRefresh_Click(object sender, EventArgs e)
        {
            Outlook._Inspector currInsp = null;
            Outlook._MailItem mail = null;
            object item = null;

            try
            {
                currInsp = this.InspectorObj as Outlook.Inspector;
                item = currInsp.CurrentItem;
                if (item is Outlook._MailItem)
                {
                    mail = item as Outlook._MailItem;
                    RefreshListBox(mail);
                }
            }
            finally
            {
                if (item != null) Marshal.ReleaseComObject(item);
                if (currInsp != null) Marshal.ReleaseComObject(currInsp);
            }
        }
    }
}
